@@ -20,6 +20,7 @@ import (
 	"k8s.io/klog"
 	utilnet "k8s.io/utils/net"
 
+	homaster "github.com/ovn-org/ovn-kubernetes/go-controller/hybrid-overlay/pkg/controller"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/metrics"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
@@ -163,6 +164,13 @@ func (oc *Controller) StartClusterMaster(masterNodeName string) error {
 		return err
 	}
 
+	if config.HybridOverlay.Enabled {
+		if err := homaster.StartMaster(oc.kube, oc.watchFactory); err != nil {
+			klog.Errorf("Failed to set up hybrid overlay master: %v", err)
+			return err
+		}
+	}
+
 	return nil
 }
 
@@ -441,8 +449,9 @@ func (oc *Controller) ensureNodeLogicalNetwork(nodeName string, hostsubnet *net.
 	clusterRouter := util.GetK8sClusterRouter()
 
 	// Create a router port and provide it the first address on the node's host subnet
-	_, stderr, err := util.RunOVNNbctl("--may-exist", "lrp-add", clusterRouter, "rtos-"+nodeName,
-		nodeLRPMAC.String(), gwIfAddr.String())
+	_, stderr, err := util.RunOVNNbctl(
+		"--if-exists", "lrp-del", "rtos-"+nodeName,
+		"--", "lrp-add", clusterRouter, "rtos-"+nodeName, nodeLRPMAC.String(), gwIfAddr.String())
 	if err != nil {
 		klog.Errorf("Failed to add logical port to router, stderr: %q, error: %v", stderr, err)
 		return err
